package ftcomputing.robo;
//-------------------------------------------------------------------------------
//
// FishFace.java                                        Ulrich Mller    20.02.09
//                                                      Aspire Eclipse 3.4 Java 6
// Funktionen zum Betrieb der fischertechnik Interfaces
//     Basis ist die umFish40.DLL v4.3.75.0 mit Wrapper.DLL javaFish40.DLL und der 
//     zugehrenden Klasse JavaFish, von der die Klasse FishFace abgeleitet ist.
//     Der Funktionsinhalt orientiert sich an der Klasse FishFace der 
//     C# Assembly FishFace2005.DLL
//
// -------------------------------------------------------------------------------

public class FishFace extends JavaFish {
	private final int jrError        = 0xE0000001;
	private final int jrPollInterval = 12;
	private final int notOpen        = 999;

	private int     iHandle      = notOpen;
	private boolean notHalt      = true;
	private int     distanceMode = Distance.Off; 
	
	public class DeviceData{
		public String Name;
		public int    Type;
		public int    SerialNr;
		public String Firmware;
	}

	public FishFace() {
		super();
	}

	public static String Version() {
		return "4.3.75.3";
	}
	
	public DeviceData getActDevice(){
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.getActDevice"));
		DeviceData dd = new DeviceData();
		dd.Name = "ftROBO_USB";
		dd.Type = jrGetActDeviceType(iHandle);
		dd.SerialNr = jrGetActDeviceSerialNr(iHandle);
		int f = jrGetActDeviceFirmwareNr(iHandle);
		dd.Firmware = String.format("%d.%d.%d.%d", 
				(f & 0xFF000000) >> 24,
				(f & 0xFF0000) >> 16,
				(f & 0xFF00) >> 8,
				(f & 0xFF)); 
		return dd;
	}
	
	public int getAnalog(int AnalogNr) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.getAnalog"));
		int res = jrGetAnalog(iHandle, AnalogNr);
		if (res == jrError)
			throw (new FishFaceException("InterfaceProblem.getAnalog"));
		return res;
	}
	
	public int  getCounter(int InputNr) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.getCounter"));
		int res = jrGetCounter(iHandle, InputNr);
		if (res == jrError)
			throw (new FishFaceException("InterfaceProblem.getCounter"));
		return res;
	}
	public void setCounter(int InputNr, int ICount) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.setCounter"));
		if (jrSetCounter(iHandle, InputNr, ICount) == jrError)
			throw (new FishFaceException("InterfaceProblem.setCounter"));
	}
	
	public int getDistance(int SensorNr) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.getDistance"));
		if (distanceMode != Distance.UltraSonic)
			throw (new FishFaceException("Kein DistanceMode.getDistance"));
		int res = jrGetDistanceValue(iHandle, SensorNr);
		if (res == jrError)
			throw (new FishFaceException("InterfaceProblem.getDistance"));
		return res;		
	}
	
	public int getDistanceMode() {
		return distanceMode;
	}
	
	public boolean getInput(int InputNr) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.getInput"));
		int res = jrGetInput(iHandle, InputNr);
		if (res == jrError)
			throw (new FishFaceException("InterfaceProblem.getInput"));
		return (res == 1) ? true : false;
	}

	public int getInputs() {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.getInputs"));
		int res = jrGetInputs(iHandle);
		if (res == jrError)
			throw (new FishFaceException("InterfaceProblem.getInputs"));
		return res;
	}
	
	public boolean getIRKey(int Code, int KeyNr) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.getIRKey"));
		int res = jrGetIRKey(iHandle, Code, KeyNr);
		if (res == jrError)
			throw (new FishFaceException("InterfaceProblem.getIRKey"));
		if (res == 1)
			return true;
		else
			return false;
	}
	
	public void setLamp(int LampNr, int OnOff) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.setLamp"));
		if (jrSetLamp(iHandle, LampNr, OnOff) == jrError)
			throw (new FishFaceException("InterfaceProblem.setLamp"));
	}
	
	public void setMotor(int MotorNr, int Direction) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.setMotor"));
		if (jrSetMotor(iHandle, MotorNr, Direction) == jrError)
			throw (new FishFaceException("InterfaceProblem.setMotor"));
	}
	public void setMotor(int MotorNr, int Direction, int Speed) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.setMotor"));
		if (jrSetMotorEx(iHandle, MotorNr, Direction, Speed) == jrError)
			throw (new FishFaceException("InterfaceProblem.setMotor"));
	}
	public void setMotor(int MotorNr, int Direction, int Speed, int ICount) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.setMotor"));
		if (jrRobMotor(iHandle, MotorNr, Direction, Speed, ICount) == jrError)
			throw (new FishFaceException("InterfaceProblem.setMotor"));
	}

	public void setMotors(int MotorStatus) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.setMotors"));
		if (jrSetMotors(iHandle, MotorStatus) == jrError)
			throw (new FishFaceException("InterfaceProblem.setMotors"));
	}

	public boolean getNotHalt() {
		return notHalt;
	}
	public void    setNotHalt(boolean OnOff) {
		notHalt = OnOff;
	}

	public int getOutputs() {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.getOutputs"));
		int res = jrGetMotors(iHandle);
		if (res == jrError)
			throw (new FishFaceException("InterfaceProblem.getOutputs"));
		return res;
	}
	
	public int getVoltage(int VoltNr) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.GetVoltage"));
		int res = jrGetVoltage(iHandle, VoltNr);
		if (res == jrError)
			throw (new FishFaceException("InterfaceProblem.GetVoltage"));
		return res;
	}

	public void clearCounter(int InputNr) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.ClearCounter"));
		if (jrSetCounter(iHandle, InputNr, 0) == jrError)
			throw (new FishFaceException("InterfaceProblem.ClearCounter"));
	}

	public void clearCounters() {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.ClearCounters"));
		if (jrClearCounters(iHandle) == jrError)
			throw (new FishFaceException("InterfaceProblem.ClearCounters"));
	}

	public void clearMotors() {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.ClearMotors"));
		if (jrSetMotors(iHandle, 0) == jrError)
			throw (new FishFaceException("InterfaceProblem.ClearMotors"));
	}

	public void closeInterface() {
		if (iHandle != notOpen)
			if (jrCloseInterface(iHandle) == jrError)
				throw (new FishFaceException("InterfaceProblem.CloseInterface"));
		iHandle = notOpen;
		distanceMode = Distance.Off;
	}

	public boolean finish() {
		return finish(0);
	}
	public boolean finish(int InputNr) {
		Thread.yield();
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.Finish"));
		if (notHalt || escape() != 0)
			return true;
		if (jrGetInputs(iHandle) == jrError)
			throw (new FishFaceException("InterfaceProblem.Finish"));
		if (InputNr > 0 && getInput(InputNr))
			return true;
		return false;
	}

	public void openInterface(int ifTyp, int SerialNr) throws FishFaceException {
		iHandle = jrOpenInterfaceUSB(ifTyp, SerialNr);
		if (iHandle == jrError)
			throw new FishFaceException("InterfaceProblem.OpenInterface");
		jrSetMotors(iHandle, 0);
		notHalt = false;
		distanceMode = Distance.UltraSonic;
	}
	
	public void openInterface(int ifTyp, int SerialNr, int DistanceMode) {
		iHandle = jrOpenInterfaceUSBdis(ifTyp, SerialNr, DistanceMode);
		if (iHandle == jrError)
			throw new FishFaceException("InterfaceProblem.OpenInterface");
		jrSetMotors(iHandle, 0);
		notHalt = false;
		distanceMode = DistanceMode;
	}

	public void pause(int mSek) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.Pause"));
		int sZeit = getTickCount();
		do {
			Thread.yield();
			if (notHalt || escape() != 0)
				return;
			if (jrGetInputs(iHandle) == jrError)
				throw (new FishFaceException("InterfaceProblem.WaitForInput"));
			try{Thread.sleep(jrPollInterval);}catch(InterruptedException ei){}
		} while (getTickCount() - sZeit < mSek);
	}

	public void waitForChange(int InputNr, int NrOfChanges) {
		waitForChange(InputNr, NrOfChanges, 0);
	}
	public void waitForChange(int InputNr, int NrOfChanges, int TermInputNr) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.WaitForChange"));
		jrSetCounter(iHandle, InputNr, 0);
		while (jrGetCounter(iHandle, InputNr) < NrOfChanges) {
			if (notHalt || escape() != 0)
				return;
			if (TermInputNr > 0)
				if (getInput(TermInputNr))
					return;
			if (jrGetInputs(iHandle) == jrError)
				throw (new FishFaceException("InterfaceProblem.WaitForInput"));
			try{Thread.sleep(jrPollInterval);}catch(InterruptedException ei){}
		}
	}

	public void waitForHigh(int InputNr){
		waitForInput(InputNr, false);
		waitForInput(InputNr, true);
	}
	
	public void waitForInput(int InputNr) {
		waitForInput(InputNr, true);
	}
	public void waitForInput(int InputNr, boolean OnOff) {
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.WaitForInput"));
		do {
			if (notHalt || escape() != 0)
				return;
			if (jrGetInputs(iHandle) == jrError)
				throw (new FishFaceException("InterfaceProblem.WaitForInput"));
			try{Thread.sleep(jrPollInterval);}catch(InterruptedException ei){}
		} while (!getInput(InputNr) == OnOff);
	}

	public void waitForLow(int InputNr) {
		waitForInput(InputNr, true);
		waitForInput(InputNr, false);
	}
	
	public int waitForMotors(int mSek, int... MotorNrs) {
		int state;
		if (iHandle == notOpen)
			throw (new FishFaceException("KeinOpen.WaitForMotors"));
		int time = getTickCount() + mSek;
		do {
			state = 0;
			for(int motNr : MotorNrs) {
				state += jrGetModeStatus(iHandle, motNr);
			}
			if(state == 0) return Wait.End;
			if(notHalt || (escape() != 0))
				for(int motNr : MotorNrs) jrSetMotor(iHandle, motNr, 0);
			if(notHalt) return Wait.NotHalt;
			if(escape() != 0) return Wait.ESC;
			try{Thread.sleep(jrPollInterval);}catch(InterruptedException ei){}
			if (jrGetInputs(iHandle) == jrError)
				throw (new FishFaceException("InterfaceProblem.WaitForMotors"));
    	} while (( mSek == 0) || time > getTickCount());
		return Wait.Time;
	}
}
